/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.buttons.music

import dev.kord.common.Color
import dev.kord.core.behavior.interaction.response.DeferredPublicMessageInteractionResponseBehavior
import dev.kord.core.behavior.interaction.response.respond
import dev.kord.core.entity.Guild
import dev.kord.core.entity.User
import dev.kord.rest.builder.component.ActionRowBuilder
import dev.kord.rest.builder.message.actionRow
import dev.kord.rest.builder.message.embed
import dev.kord.rest.builder.message.modify.actionRow
import dev.schlaubi.lavakord.audio.Link
import net.moonleay.botendo.extensions.music.components.MusicManager
import net.moonleay.botendo.util.ButtonUtil
import net.moonleay.botendo.util.MessageUtil
import net.moonleay.botendo.util.UserUtil

class RepeatButton : net.moonleay.botendo.buttons.Button("btn.music.repeat") {
    override suspend fun onInteraction(
        response: DeferredPublicMessageInteractionResponseBehavior,
        guild: Guild,
        user: User
    ) {
        val guildId = guild.id
        val link = net.moonleay.botendo.Bot.lava.getLink(guildId.toString())
        val player = link.player
        val voiceState = user.asMember(guildId).getVoiceStateOrNull()
        if (voiceState == null) {
            response.respond {
                this.embed {
                    this.color = Color(0xE0311A)
                    this.title = "You are not connected to a VC"
                    this.description = "Please connect to a VC"
                    this.footer {
                        this.text = MessageUtil.getFooter(user)
                    }
                }
            }
            return
        }

        val channelId = voiceState.channelId
        if (link.state == Link.State.NOT_CONNECTED) {
            response.respond {
                this.embed {
                    this.color = Color(0xE0311A)
                    this.title = "Not connected"
                    this.description = "I'm not in a VC and therefore, I am not playing anything."
                    this.footer {
                        this.text = MessageUtil.getFooter(user)
                    }
                }
            }
            return
        } else if (link.state == Link.State.CONNECTED && link.lastChannelId != channelId!!.value) {
            response.respond {
                this.embed {
                    this.color = Color(0xE0311A)
                    this.title = "You are not in my VC"
                    this.description = "We are not in the same VC and therefore, you cannot control the music"
                    this.footer {
                        this.text = MessageUtil.getFooter(user)
                    }
                }
            }
            return
        }

        val gts = MusicManager.getGuildTrackScheduler(guild, player)
        gts.repeating = !gts.repeating
        response.respond {
            this.embed {
                this.color = Color(0x52E01A)
                this.title = if (gts.repeating) "Now Repeating" else "Now Continuing"
                this.description =
                    if (gts.repeating) "The current track will now loop" else "The next track will play when this song finishes"
                this.footer {
                    this.text = MessageUtil.getFooter(user)
                }
            }

            this.actionRow(fun ActionRowBuilder.() {
                components.addAll(ButtonUtil.getMusicControllerButtons(player.paused, gts.repeating).components)
            })
        }
    }
}
