/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.extensions.music

import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import dev.kord.common.Color
import dev.kord.rest.builder.component.ActionRowBuilder
import dev.kord.rest.builder.message.actionRow
import dev.kord.rest.builder.message.create.actionRow
import dev.kord.rest.builder.message.embed
import dev.schlaubi.lavakord.audio.Link
import dev.schlaubi.lavakord.kord.getLink
import net.moonleay.botendo.extensions.music.components.MusicManager
import net.moonleay.botendo.util.*

class NowPlayingExtension : Extension() {
    override val name = "nowplaying"
    override suspend fun setup() {
        publicSlashCommand {
            name = "nowplaying"
            description = "Show what's currently playing"
            this.action {
                val guildId = this.guild!!.id
                val link = net.moonleay.botendo.Bot.lava.getLink(guildId)
                val player = link.player
                val u = this.user
                if (link.state == Link.State.NOT_CONNECTED) {
                    this.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "Not connected"
                            this.description = "I'm not in a VC and therefore, I am not playing anything."
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
                    return@action
                }
                val track = player.playingTrack
                if (track == null) {
                    this.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "Not Playing"
                            this.description = "I'm not playing anything currently"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
                    return@action
                }
                val gts = MusicManager.getGuildTrackScheduler(this.guild!!.asGuild(), player)

                this.respond {
                    this.embed {
                        this.color = Color(0x52E01A)
                        this.title = "Currently playing"
                        this.description = "**${track.info.title}**\n*Now Playing*\nby ${track.info.author} ;    ${
                            TimeUtil.getTimeFormatedRaw(
                                player.position
                            )
                        }: ${
                            TimeUtil.getTimeFormatedRaw(
                                track.info.length
                            )
                        }\n" +
                                ">>>${track.info.uri}"
                        this.footer {
                            this.text = MessageUtil.getFooter(u)
                        }
                        if(track.info.uri == null){
                            this.thumbnail {
                                this.url = "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(track.info.uri!!) + "/maxresdefault.jpg"
                            }
                        }
                    }

                    this.actionRow(fun ActionRowBuilder.() {
                        components.addAll(
                            ButtonUtil.getMusicControllerButtons(
                                player.paused,
                                gts.repeating
                            ).components
                        )
                    })
                }
            }
        }
    }
}
