/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.extensions.music

import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import dev.kord.common.Color
import dev.kord.rest.builder.message.embed
import dev.schlaubi.lavakord.audio.Link
import dev.schlaubi.lavakord.kord.getLink
import net.moonleay.botendo.extensions.music.components.LinkArguments
import net.moonleay.botendo.extensions.music.components.MusicManager
import net.moonleay.botendo.util.MessageUtil
import net.moonleay.botendo.util.UserUtil


class UpsertExtension : Extension() {

    override val name = "upsert"

    override suspend fun setup() {
        publicSlashCommand(::LinkArguments) {
            name = "upsert"
            description = "Upsert music"
            this.action {
                val guildId = this.guild!!.id
                val link = net.moonleay.botendo.Bot.lava.getLink(guildId)
                val u = this.user
                val vcsUser = u.asMember(guildId).getVoiceStateOrNull()
                if (vcsUser == null) {
                    this.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "You are not connected to a VC"
                            this.description = "Please connect to a VC"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
                    return@action
                }

                val channelId = vcsUser.channelId

                if (link.state != Link.State.CONNECTED) {
                    link.connectAudio(channelId!!.value)
                } else if (link.state == Link.State.CONNECTED && link.lastChannelId != channelId!!.value) {
                    this.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "You are not in my VC"
                            this.description = "We are not in the same VC and therefore, you cannot play any music"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
                    return@action
                }

                val query = arguments.linkquery
                val search = if (query.startsWith("http")) {
                    query
                } else {
                    "ytsearch:$query"
                }
                this.respond {
                    this.embed {
                        this.color = Color(0xE0A81A)
                        this.title = "Searching..."
                        this.description = "We are looking for $query"
                        this.footer {
                            this.text = MessageUtil.getFooter(u)
                        }
                    }
                }
                MusicManager.upsertIntoQueue(this, link, search)
            }
        }
    }
}

